#
# inst.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

require 'tempfile'

require 'amstd/futils'
require 'amstd/rbparams'
require 'amstd/bug'


class Installer

  extend FileUtils
  include FileUtils

	# sync init value
	LongOpts = %w( rb-dir so-dir ruby-path bin-dir )

	OptExp = /\A--(#{LongOpts.join '|'})=/o


  def initialize( argv = nil )
    @verbose = true
		@opts = {}
		@opts['bin-dir']   = RubyParams::BINDIR
		@opts['rb-dir']    = RubyParams::RBDIR
		@opts['so-dir']    = RubyParams::SODIR
		@opts['ruby-path'] = RubyParams::RUBY_PATH
		@task = nil
		@wrong_arg = []

		argv ||= ARGV
		argv.delete_if do |i|
		  case i
			when OptExp
			  name = $1
				ar   = $'.strip

				unless @opts[ name ] then
				  raise ArgumentError, "unknown argument '--#{name}'"
				end
				@opts[ name ] = ar
				true

			when 'setup', 'install', 'setup-install'
				if @task then
				  raise ArgumentError, "too many # of task: '#{@task}' and '#{i}'"
				end
				@task = i
				true

			when '-h', '--help'
				print_usage
				exit 0

			else
				@wrong_arg.push i.dup
				false
			end
		end

		unless @task then
		  raise ArgumentError, "task not given: try 'ruby setup.rb --help'"
		end
  end

	attr :opts
	attr :task
	attr :wrong_arg

  def print_usage
		print <<S

usage:

  ruby setup.rb [<parametors>] setup|install|setup-install

parametors:

  --rb-dir=dir     [<RUBY LIB>/site_ruby/]
  --so-dir=dir     [<RUBY LIB>/site_ruby/<ARCH>]
	--ruby-path=dir  [<path for THIS ruby>]
	--bin-dir=dir    [<path for THIS ruby>]

S
  end


	class << self

		def execute
			begin
				i = new( ARGV )
			rescue
raise
			  puts $!
				exit 1
			end
			unless i.wrong_arg.empty? then
			  puts "unknown arguments '#{i.wrong_arg.join %<', '>}'"
				exit 1
			end
			i.execute
		end

	end

  def execute
		case @task
		when 'setup-install'
      try_setup
      try_install

		when 'setup'
      try_setup

    when 'install'
		  try_install

    else
		  bug!
    end
  end

  def try_setup
		puts 'entering setup phase...'
    begin
      com_setup
    rescue
      puts 'setup failed'
      raise
    end
		puts 'setup done.'
  end

  def try_install
		puts 'entering install phase...'
    begin
      com_install
    rescue
      puts 'install failed'
      raise
    end
		puts 'installation done.'
  end


	def into_dir( libn )
	  unless directory? libn then
		  # for raw package
			puts "inst.rb: skip #{libn} (not critical)"
			return
		end
		chdir( libn ) do
			yield
		end
	end


	def install_bin( fn )
	  install fn, isdir( @opts['bin-dir'] ), 0755
	end


  def setup_library( libn )
    compile_all_in libn
  end

  def install_library( libn )
    lib_install( libn ) do |rb_to, so_to|
      install_rb rb_to
      each_dir do |ext|
        if so_dir? '.' then
          make_install so_to
        end
      end
    end
  end

  def lib_install( libname )
    into_dir( libname ) do
      yield fjoin(@opts['rb-dir'], libname),
            fjoin(@opts['so-dir'], libname)
    end
  end


  def install_rb( to = nil )
		to ||= @opts['rb-dir']
    Dir.foreach( '.' ) do |fn|
      if /\.rb\z/o === fn and file? fn then
        install fn, isdir( to ), 0644
      end
    end
  end


  def extconf
    unless is_newer? 'Makefile', 'extconf.rb' then
      system 'ruby extconf.rb'
    end
  end

  def make
    command 'make'
  end

  def compile_all_in( dir )
    chdir( dir ) do
      each_sodir do |extname|
        extconf
        make
      end
    end
  end

  def make_install( to = nil )
		to ||= @opts['so-dir']
		isdir to
    find_so('.').each do |fn|
      install fn, to, 0555
    end
  end

  def find_so( dir = '.' )
    fnames = nil
    Dir.open( dir ) {|d| fnames = d.to_a }
    %w( so sl dll o ).each do |ext|
      exp = /\.#{ext}\z/
      a = fnames.find_all {|f| exp === f }
      return a if Array === a
    end
    
    raise ArgumentError,
      "can't find ruby extention library in '#{expand dir}'"
  end

  def so_dir?( dn = '.' )
    file? fjoin( dn, 'MANIFEST' )
  end

  def each_sodir
    each_dir do |dn|
      if so_dir? then
        yield dn
      end
    end
  end
        
  def each_dir
    Dir.foreach( '.' ) do |fn|
      next if /\A\./o === fn
      if directory? fn then
        chdir( fn ) do
          yield fn
        end
      end
    end
  end


  def install_all( to )
    Dir.foreach( '.' ) do |fn|
      if file? fn then
        install fn, to, 644
      end
    end
  end


  def add_rubypath( fn, opt = '' )
    opt = ' ' + opt unless opt.empty?

    tmpf = Tempfile.open( 'rbinst' )
    File.open( fn ) do |f|
      first = f.gets
      tmpf.puts "\#!#{@opts['ruby-path']}#{opt}"
      tmpf << first unless /\A\#\!/o === first
      f.each {|i| tmpf << i }
    end
    tmpf.close
    
    mod = File.stat( fn ).mode
    tmpf.open
    open( fn, 'w' ) do |wf|
      tmpf.each {|i| wf << i }
    end
    chmod mod, fn

    tmpf.close true
  end

end
